@@ -97,8 +97,9 @@ class Nanny(ServerNode):
                                     connection_args=self.connection_args,
                                     **kwargs)
 
-        pc = PeriodicCallback(self.memory_monitor, 100)
-        self.periodic_callbacks['memory'] = pc
+        if self.memory_limit:
+            pc = PeriodicCallback(self.memory_monitor, 100)
+            self.periodic_callbacks['memory'] = pc
 
         self._listen_address = listen_address
         self.status = 'init'
@@ -157,8 +158,7 @@ class Nanny(ServerNode):
             assert self.worker_address
             self.status = 'running'
 
-        for pc in self.periodic_callbacks.values():
-            pc.start()
+        self.start_periodic_callbacks()
 
     def start(self, addr_or_port=0):
         self.loop.add_callback(self._start, addr_or_port)
