@@ -225,3 +225,14 @@ def test_Future_knows_status_immediately(c, s, a, b):
             yield gen.sleep(0.05)
 
     yield c2.close()
+
+
+@gen_cluster(client=True)
+def test_erred_future(c, s, a, b):
+    future = c.submit(div, 1, 0)
+    q = Queue()
+    yield q.put(future)
+    yield gen.sleep(0.1)
+    future2 = yield q.get()
+    with pytest.raises(ZeroDivisionError):
+        yield future2.result()
