@@ -17,12 +17,18 @@ from distributed.metrics import time
 from distributed.scheduler import BANDWIDTH, key_split
 from distributed.utils_test import (slowinc, slowadd, inc, gen_cluster,
                                     slowidentity)
-from distributed.utils_test import loop # flake8: noqa
+from distributed.utils_test import (loop, nodebug_setup_module,
+                                    nodebug_teardown_module)  # flake8: noqa
 from distributed.worker import TOTAL_MEMORY
 
 import pytest
 
 
+# Most tests here are timing-dependent
+setup_module = nodebug_setup_module
+teardown_module = nodebug_teardown_module
+
+
 @pytest.mark.skipif(not sys.platform.startswith('linux'),
                     reason="Need 127.0.0.2 to mean localhost")
 @gen_cluster(client=True, ncores=[('127.0.0.1', 2), ('127.0.0.2', 2)],
@@ -34,7 +40,6 @@ def test_work_stealing(c, s, a, b):
     yield wait(futures)
     assert len(a.data) > 10
     assert len(b.data) > 10
-    assert len(a.data) > len(b.data) - 5
 
 
 @gen_cluster(client=True, ncores=[('127.0.0.1', 1)] * 2)
@@ -331,18 +336,21 @@ def test_steal_when_more_tasks(c, s, a, *rest):
 
 @gen_cluster(client=True, ncores=[('127.0.0.1', 1)] * 10)
 def test_steal_more_attractive_tasks(c, s, a, *rest):
+
     def slow2(x):
         sleep(1)
         return x
+
     s.extensions['stealing']._pc.callback_time = 20
     x = c.submit(mul, b'0', 100000000, workers=a.address)  # 100 MB
     yield wait(x)
+
     s.task_duration['slowidentity'] = 0.2
     s.task_duration['slow2'] = 1
 
-    future = c.submit(slow2, x)
     futures = [c.submit(slowidentity, x, pure=False, delay=0.2)
                for i in range(10)]
+    future = c.submit(slow2, x)
 
     while not any(w.task_state for w in rest):
         yield gen.sleep(0.01)
@@ -366,7 +374,7 @@ def assert_balanced(inp, expected, c, s, *workers):
 
     futures = []
     for w, ts in zip(workers, inp):
-        for t in ts:
+        for t in sorted(ts, reverse=True):
             if t:
                 [dat] = yield c._scatter([next(data_seq)], workers=w.address)
                 s.nbytes[dat.key] = BANDWIDTH * t
@@ -381,21 +389,28 @@ def assert_balanced(inp, expected, c, s, *workers):
     while len(s.rprocessing) < len(futures):
         yield gen.sleep(0.001)
 
-    s.extensions['stealing'].balance()
 
-    result = [sorted([int(key_split(k)) for k in s.processing[w.address]],
-                     reverse=True)
-              for w in workers]
+    for i in range(10):
+        steal.balance()
 
-    result2 = sorted(result, reverse=True)
-    expected2 = sorted(expected, reverse=True)
+        while steal.in_flight:
+            yield gen.sleep(0.001)
 
-    if config.get('pdb-on-err'):
-        if result2 != expected2:
-            import pdb
-            pdb.set_trace()
+        result = [sorted([int(key_split(k)) for k in s.processing[w.address]],
+                         reverse=True)
+                  for w in workers]
 
-    assert result2 == expected2
+        result2 = sorted(result, reverse=True)
+        expected2 = sorted(expected, reverse=True)
+
+        if config.get('pdb-on-err'):
+            if result2 != expected2:
+                import pdb
+                pdb.set_trace()
+
+        if result2 == expected2:
+            return
+    raise Exception('Expected: {}; got: {}'.format(str(expected2), str(result2)))
 
 
 @pytest.mark.parametrize('inp,expected', [
@@ -446,12 +461,12 @@ def assert_balanced(inp, expected, c, s, *workers):
       [1],
       [1]]),
 
-    ([[1, 1, 1, 1, 1, 1, 1],
+    pytest.mark.xfail(([[1, 1, 1, 1, 1, 1, 1],
       [1, 1], [1, 1], [1, 1],
       []],
      [[1, 1, 1, 1, 1],
       [1, 1], [1, 1], [1, 1],
-      [1, 1]])
+      [1, 1]]), reason="Some uncertainty based on executing stolen task")
 ])
 def test_balance(inp, expected):
     test = lambda *args, **kwargs: assert_balanced(inp, expected, *args, **kwargs)
@@ -478,6 +493,7 @@ def test_restart(c, s, a, b):
 
 @gen_cluster(client=True)
 def test_steal_communication_heavy_tasks(c, s, a, b):
+    steal = s.extensions['stealing']
     s.task_duration['slowadd'] = 0.001
     x = c.submit(mul, b'0', int(BANDWIDTH), workers=a.address)
     y = c.submit(mul, b'1', int(BANDWIDTH), workers=b.address)
@@ -489,7 +505,9 @@ def test_steal_communication_heavy_tasks(c, s, a, b):
     while not any(f.key in s.rprocessing for f in futures):
         yield gen.sleep(0.01)
 
-    s.extensions['stealing'].balance()
+    steal.balance()
+    while steal.in_flight:
+        yield gen.sleep(0.001)
 
     assert s.processing[b.address]
 
@@ -504,33 +522,34 @@ def test_steal_twice(c, s, a, b):
     while len(s.task_state) < 100:  # tasks are all allocated
         yield gen.sleep(0.01)
 
-    workers = [Worker(s.ip, s.port, loop=s.loop) for _ in range(30)]
+    workers = [Worker(s.ip, s.port, loop=s.loop) for _ in range(20)]
     yield [w._start() for w in workers]  # army of new workers arrives to help
 
     yield wait(futures)
 
-    assert all(s.has_what.values())
-    assert max(map(len, s.has_what.values())) < 20
+    has_what = dict(s.has_what)  # take snapshot
+    empty_workers = [w for w, keys in has_what.items() if not len(keys)]
+    if len(empty_workers) > 2:
+        pytest.fail("Too many workers without keys (%d out of %d)"
+                    % (len(empty_workers), len(has_what)))
+    assert max(map(len, has_what.values())) < 30
 
+    yield c._close()
     yield [w._close() for w in workers]
 
 
 @gen_cluster(client=True)
-def test_accept_old_result_if_stolen(c, s, a, b):
+def test_dont_steal_executing_tasks(c, s, a, b):
+    steal = s.extensions['stealing']
+
     future = c.submit(slowinc, 1, delay=0.5, workers=a.address)
     while not a.executing:
         yield gen.sleep(0.01)
-    steal = s.extensions['stealing']
 
-    yield gen.sleep(0.25)
-
-    steal.move_task(future.key, a.address, b.address)
-    while not b.executing:
-        yield gen.sleep(0.01)
-
-    yield gen.sleep(0.35)
-
-    assert future.key in s.who_has
+    steal.move_task_request(future.key, a.address, b.address)
+    yield gen.sleep(0.1)
+    assert future.key in a.executing
+    assert not b.executing
 
 
 @gen_cluster(client=True, ncores=[('127.0.0.1', 1)] * 2)
