@@ -19,7 +19,7 @@ from tornado import gen
 from tornado.ioloop import TimeoutError
 
 from distributed import (Nanny, Client, get_client, wait, default_client,
-        get_worker)
+        get_worker, Reschedule)
 from distributed.compatibility import WINDOWS
 from distributed.core import rpc
 from distributed.client import wait
@@ -30,7 +30,7 @@ from distributed.utils import tmpfile
 from distributed.utils_test import (inc, mul, gen_cluster, div, dec,
                                     slow, slowinc, gen_test, cluster,
                                     captured_logger)
-from distributed.utils_test import loop # flake8: noqa
+from distributed.utils_test import loop, nodebug # flake8: noqa
 
 
 def test_worker_ncores():
@@ -318,20 +318,21 @@ def test_io_loop(loop):
 
 
 @gen_cluster(client=True, ncores=[])
-def test_spill_to_disk(e, s):
+def test_spill_to_disk(c, s):
     np = pytest.importorskip('numpy')
-    w = Worker(s.address, loop=s.loop, memory_limit=1200 / 0.6)
+    w = Worker(s.address, loop=s.loop, memory_limit=1200 / 0.6,
+               memory_pause_fraction=None, memory_spill_fraction=None)
     yield w._start()
 
-    x = e.submit(np.random.randint, 0, 255, size=500, dtype='u1', key='x')
+    x = c.submit(np.random.randint, 0, 255, size=500, dtype='u1', key='x')
     yield wait(x)
-    y = e.submit(np.random.randint, 0, 255, size=500, dtype='u1', key='y')
+    y = c.submit(np.random.randint, 0, 255, size=500, dtype='u1', key='y')
     yield wait(y)
 
     assert set(w.data) == {x.key, y.key}
     assert set(w.data.fast) == {x.key, y.key}
 
-    z = e.submit(np.random.randint, 0, 255, size=500, dtype='u1', key='z')
+    z = c.submit(np.random.randint, 0, 255, size=500, dtype='u1', key='z')
     yield wait(z)
     assert set(w.data) == {x.key, y.key, z.key}
     assert set(w.data.fast) == {y.key, z.key}
@@ -909,13 +910,14 @@ def test_worker_fds(s):
                     reason="Need 127.0.0.2 to mean localhost")
 @gen_cluster(ncores=[])
 def test_service_hosts_match_worker(s):
-    from distributed.http.worker import HTTPWorker
-    services = {('http', 0): HTTPWorker}
+    pytest.importorskip('bokeh')
+    from distributed.bokeh.worker import BokehWorker
+    services = {('bokeh', 0): BokehWorker}
     for host in ['tcp://0.0.0.0', 'tcp://127.0.0.2']:
         w = Worker(s.address, services=services)
         yield w._start(host)
 
-        sock = first(w.services['http']._sockets.values())
+        sock = first(w.services['bokeh'].server._http._sockets.values())
         assert sock.getsockname()[0] == host.split('://')[1]
         yield w._close()
 
@@ -949,9 +951,9 @@ def test_statistical_profiling(c, s, a, b):
 
     profile = a.profile_keys['slowinc']
     assert profile['count']
-    assert 'threading' not in str(profile)
 
 
+@nodebug
 @gen_cluster(client=True)
 def test_statistical_profiling_2(c, s, a, b):
     da = pytest.importorskip('dask.array')
@@ -961,9 +963,7 @@ def test_statistical_profiling_2(c, s, a, b):
         yield wait(y)
     profile = a.get_profile()
     assert profile['count']
-    assert 'threading' not in str(profile)
-    assert 'sum' in str(profile)
-    assert 'random' in str(profile)
+    assert 'sum' in str(profile) or 'random' in str(profile)
 
 
 @gen_cluster(ncores=[('127.0.0.1', 1)], client=True, worker_kwargs={'memory_monitor_interval': 10})
@@ -1013,7 +1013,7 @@ def test_pause_executor(c, s, a):
         assert a.paused
         out = logger.getvalue()
         assert 'memory' in out.lower()
-        assert 'stop' in out.lower()
+        assert 'pausing' in out.lower()
 
     assert sum(f.status == 'finished' for f in futures) < 4
 
@@ -1045,3 +1045,45 @@ def test_get_current_task(c, s, a, b):
 
     result = yield c.submit(some_name)
     assert result.startswith('some_name')
+
+
+@gen_cluster(client=True, ncores=[('127.0.0.1', 1)] * 2)
+def test_reschedule(c, s, a, b):
+    s.extensions['stealing']._pc.stop()
+    a_address = a.address
+    def f(x):
+        sleep(0.1)
+        if get_worker().address == a_address:
+            raise Reschedule()
+
+    futures = c.map(f, range(4))
+    futures2 = c.map(slowinc, range(10), delay=0.1, workers=a.address)
+    yield wait(futures)
+
+    assert all(f.key in b.data for f in futures)
+
+
+def test_deque_handler():
+    from distributed.worker import deque_handler, logger
+    logger.info('foo456')
+    assert deque_handler.deque
+    msg = deque_handler.deque[-1]
+    assert 'distributed.worker' in deque_handler.format(msg)
+    assert any(msg.msg == 'foo456' for msg in deque_handler.deque)
+
+
+@gen_cluster(ncores=[], client=True)
+def test_avoid_memory_monitor_if_zero_limit(c, s):
+    worker = Worker(s.address, loop=s.loop, memory_limit=0,
+                    memory_monitor_interval=10)
+    yield worker._start()
+    assert type(worker.data) is dict
+    assert 'memory' not in worker.periodic_callbacks
+
+    future = c.submit(inc, 1)
+    assert (yield future) == 2
+    yield gen.sleep(worker.memory_monitor_interval / 1000)
+
+    yield c.submit(inc, 2)  # worker doesn't pause
+
+    yield worker._close()
