@@ -14,7 +14,6 @@ import re
 import shutil
 import signal
 import socket
-import ssl
 import subprocess
 import sys
 import tempfile
@@ -25,6 +24,11 @@ import uuid
 import warnings
 import weakref
 
+try:
+    import ssl
+except ImportError:
+    ssl = None
+
 import psutil
 import pytest
 import six
@@ -642,12 +646,6 @@ def end_cluster(s, workers):
     def end_worker(w):
         with ignoring(TimeoutError, CommClosedError, EnvironmentError):
             yield w._close(report=False)
-        if isinstance(w, Nanny):
-            dir = w.worker_dir
-        else:
-            dir = w.local_dir
-        if dir and os.path.exists(dir):
-            shutil.rmtree(dir)
 
     yield [end_worker(w) for w in workers]
     yield s.close()  # wait until scheduler stops completely
@@ -978,13 +976,16 @@ def assert_can_connect_locally_6(port, timeout=None, connection_args=None):
 
 
 @contextmanager
-def captured_logger(logger, level=logging.INFO):
+def captured_logger(logger, level=logging.INFO, propagate=None):
     """Capture output from the given Logger.
     """
     if isinstance(logger, str):
         logger = logging.getLogger(logger)
     orig_level = logger.level
     orig_handlers = logger.handlers[:]
+    if propagate is not None:
+        orig_propagate = logger.propagate
+        logger.propagate = propagate
     sio = six.StringIO()
     logger.handlers[:] = [logging.StreamHandler(sio)]
     logger.setLevel(level)
@@ -993,6 +994,8 @@ def captured_logger(logger, level=logging.INFO):
     finally:
         logger.handlers[:] = orig_handlers
         logger.setLevel(orig_level)
+        if propagate is not None:
+            logger.propagate = orig_propagate
 
 
 @contextmanager
