@@ -8,9 +8,7 @@ import logging
 import os
 from pickle import PicklingError
 import random
-import tempfile
 import threading
-import shutil
 import sys
 import weakref
 
@@ -33,6 +31,7 @@ from .config import config, log_format
 from .compatibility import unicode, get_thread_identity
 from .core import (error_message, CommClosedError,
                    rpc, pingpong, coerce_to_address)
+from .diskutils import WorkSpace
 from .metrics import time
 from .node import ServerNode
 from .preloading import preload_modules
@@ -104,9 +103,11 @@ class WorkerBase(ServerNode):
 
         if local_dir:
             local_dir = os.path.abspath(local_dir)
-            if not os.path.exists(local_dir):
-                os.mkdir(local_dir)
-        self.local_dir = tempfile.mkdtemp(prefix='worker-', dir=local_dir)
+        else:
+            local_dir = 'dask-worker-space'
+        self._workspace = WorkSpace(local_dir)
+        self._workdir = self._workspace.new_work_dir(prefix='worker-')
+        self.local_dir = self._workdir.dir_path
 
         self.security = security or Security()
         assert isinstance(self.security, Security)
@@ -368,8 +369,7 @@ class WorkerBase(ServerNode):
             logger.info('        Registered to: %26s', self.scheduler.address)
             logger.info('-' * 49)
 
-        for pc in self.periodic_callbacks.values():
-            pc.start()
+        self.start_periodic_callbacks()
 
     def start(self, port=0):
         self.loop.add_callback(self._start, port)
@@ -399,8 +399,7 @@ class WorkerBase(ServerNode):
             self.executor.shutdown(timeout=timeout)
         else:
             self.executor.shutdown(wait=False)
-        if os.path.exists(self.local_dir):
-            shutil.rmtree(self.local_dir)
+        self._workdir.release()
 
         for k, v in self.services.items():
             v.stop()
@@ -1145,7 +1144,6 @@ class Worker(WorkerBase):
 
         pc = PeriodicCallback(self.cycle_profile,
                               profile_cycle_interval)
-        pc.start()
         self.periodic_callbacks['profile-cycle'] = pc
 
         _global_workers.append(weakref.ref(self))
@@ -2627,7 +2625,10 @@ def secede():
     """
     Have this task secede from the worker's thread pool
 
-    This opens up a new scheduling slot and a new thread for a new task.
+    This opens up a new scheduling slot and a new thread for a new task. This
+    enables the client to schedule tasks on this node, which is
+    especially useful while waiting for other jobs to finish (e.g., with
+    ``client.gather``).
 
     Examples
     --------
